package docs

// @title           Aptly API
// @description.markdown

// @contact.name   Aptly
// @contact.url    http://github.com/aptly-dev/aptly

// @Tag.name Repos
// @Tag.description.markdown
// @Tag.name Files
// @Tag.description.markdown
// @Tag.name Mirrors
// @Tag.description.markdown
// @Tag.name Snapshots
// @Tag.description.markdown
// @Tag.name Publish
// @Tag.description.markdown
// @Tag.name Packages
// @Tag.description.markdown
// @Tag.name Status
// @Tag.description.markdown
// @Tag.name Database
// @Tag.description.markdown
// @Tag.name Tasks
// @Tag.description.markdown

// version will be appended here:
